from typing import overload
from enum import IntEnum
import abc
import datetime
import typing

import QuantConnect
import QuantConnect.Brokerages
import QuantConnect.Data.Market
import QuantConnect.Data.UniverseSelection
import QuantConnect.Interfaces
import QuantConnect.Lean.Engine.Results
import QuantConnect.Lean.Engine.TransactionHandlers
import QuantConnect.Logging
import QuantConnect.Orders
import QuantConnect.Packets
import QuantConnect.Securities
import QuantConnect.Statistics
import System
import System.Collections.Concurrent
import System.Collections.Generic
import System.Threading


class ResultHandlerInitializeParameters(System.Object):
    """DTO parameters class to initialize a result handler"""

    @property
    def job(self) -> QuantConnect.Packets.AlgorithmNodePacket:
        """The algorithm job"""
        ...

    @job.setter
    def job(self, value: QuantConnect.Packets.AlgorithmNodePacket) -> None:
        ...

    @property
    def messaging_handler(self) -> QuantConnect.Interfaces.IMessagingHandler:
        """The messaging handler"""
        ...

    @messaging_handler.setter
    def messaging_handler(self, value: QuantConnect.Interfaces.IMessagingHandler) -> None:
        ...

    @property
    def api(self) -> QuantConnect.Interfaces.IApi:
        """The Api instance"""
        ...

    @api.setter
    def api(self, value: QuantConnect.Interfaces.IApi) -> None:
        ...

    @property
    def transaction_handler(self) -> QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler:
        """The transaction handler"""
        ...

    @transaction_handler.setter
    def transaction_handler(self, value: QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler) -> None:
        ...

    @property
    def map_file_provider(self) -> QuantConnect.Interfaces.IMapFileProvider:
        """The map file provider instance to use"""
        ...

    @map_file_provider.setter
    def map_file_provider(self, value: QuantConnect.Interfaces.IMapFileProvider) -> None:
        ...

    def __init__(self, job: QuantConnect.Packets.AlgorithmNodePacket, messaging_handler: QuantConnect.Interfaces.IMessagingHandler, api: QuantConnect.Interfaces.IApi, transaction_handler: QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler, map_file_provider: QuantConnect.Interfaces.IMapFileProvider) -> None:
        """Creates a new instance"""
        ...


class BaseResultsHandler(System.Object, metaclass=abc.ABCMeta):
    """Provides base functionality to the implementations of IResultHandler"""

    STRATEGY_EQUITY_KEY: str = "Strategy Equity"
    """String message saying: Strategy Equity"""

    EQUITY_KEY: str = "Equity"
    """String message saying: Equity"""

    RETURN_KEY: str = "Return"
    """String message saying: Return"""

    BENCHMARK_KEY: str = "Benchmark"
    """String message saying: Benchmark"""

    DRAWDOWN_KEY: str = "Drawdown"
    """String message saying: Drawdown"""

    PORTFOLIO_TURNOVER_KEY: str = "Portfolio Turnover"
    """String message saying: PortfolioTurnover"""

    PORTFOLIO_MARGIN_KEY: str = "Portfolio Margin"
    """String message saying: Portfolio Margin"""

    ASSETS_SALES_VOLUME_KEY: str = "Assets Sales Volume"
    """String message saying: Portfolio Margin"""

    @property
    def main_update_interval(self) -> datetime.timedelta:
        """
        The main loop update interval
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def chart_update_interval(self) -> datetime.timedelta:
        """
        The chart update interval
        
        
        This codeEntityType is protected.
        """
        ...

    @chart_update_interval.setter
    def chart_update_interval(self, value: datetime.timedelta) -> None:
        ...

    @property
    def last_delta_order_position(self) -> int:
        """
        The last position consumed from the ITransactionHandler.order_events by get_delta_orders
        
        This codeEntityType is protected.
        """
        ...

    @last_delta_order_position.setter
    def last_delta_order_position(self, value: int) -> None:
        ...

    @property
    def last_delta_order_events_position(self) -> int:
        """
        The last position consumed from the ITransactionHandler.order_events while determining delta order events
        
        
        This codeEntityType is protected.
        """
        ...

    @last_delta_order_events_position.setter
    def last_delta_order_events_position(self, value: int) -> None:
        ...

    @property
    def serializer_settings(self) -> typing.Any:
        """
        Serializer settings to use
        
        
        This codeEntityType is protected.
        """
        ...

    @serializer_settings.setter
    def serializer_settings(self, value: typing.Any) -> None:
        ...

    @property
    def current_algorithm_equity(self) -> QuantConnect.Data.Market.Bar:
        """
        The current aggregated equity bar for sampling.
        It will be aggregated with values from the get_portfolio_value
        
        This codeEntityType is protected.
        """
        ...

    @current_algorithm_equity.setter
    def current_algorithm_equity(self, value: QuantConnect.Data.Market.Bar) -> None:
        ...

    @property
    def is_active(self) -> bool:
        """Boolean flag indicating the thread is still active."""
        ...

    @property
    def messages(self) -> System.Collections.Concurrent.ConcurrentQueue[QuantConnect.Packets.Packet]:
        """Live packet messaging queue. Queue the messages here and send when the result queue is ready."""
        ...

    @messages.setter
    def messages(self, value: System.Collections.Concurrent.ConcurrentQueue[QuantConnect.Packets.Packet]) -> None:
        ...

    @property
    def charts(self) -> System.Collections.Concurrent.ConcurrentDictionary[str, QuantConnect.Chart]:
        """Storage for the price and equity charts of the live results."""
        ...

    @charts.setter
    def charts(self, value: System.Collections.Concurrent.ConcurrentDictionary[str, QuantConnect.Chart]) -> None:
        ...

    @property
    def exit_triggered(self) -> bool:
        """
        True if the exit has been triggered
        
        
        This codeEntityType is protected.
        """
        ...

    @exit_triggered.setter
    def exit_triggered(self, value: bool) -> None:
        ...

    @property
    def exit_event(self) -> System.Threading.ManualResetEvent:
        """
        Event set when exit is triggered
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def log_store(self) -> typing.List[QuantConnect.Logging.LogEntry]:
        """
        The log store instance
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def algorithm_performance_charts(self) -> typing.List[str]:
        """
        Algorithms performance related chart names
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def chart_lock(self) -> System.Object:
        """
        Lock to be used when accessing the chart collection
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def project_id(self) -> int:
        """
        The algorithm project id
        
        
        This codeEntityType is protected.
        """
        ...

    @project_id.setter
    def project_id(self, value: int) -> None:
        ...

    @property
    def ram_allocation(self) -> str:
        """
        The maximum amount of RAM (in MB) this algorithm is allowed to utilize
        
        
        This codeEntityType is protected.
        """
        ...

    @ram_allocation.setter
    def ram_allocation(self, value: str) -> None:
        ...

    @property
    def compile_id(self) -> str:
        """
        The algorithm unique compilation id
        
        
        This codeEntityType is protected.
        """
        ...

    @compile_id.setter
    def compile_id(self, value: str) -> None:
        ...

    @property
    def algorithm_id(self) -> str:
        """
        The algorithm job id.
        This is the deploy id for live, backtesting id for backtesting
        
        
        This codeEntityType is protected.
        """
        ...

    @algorithm_id.setter
    def algorithm_id(self, value: str) -> None:
        ...

    @property
    def start_time(self) -> datetime.datetime:
        """
        The result handler start time
        
        
        This codeEntityType is protected.
        """
        ...

    @property
    def runtime_statistics(self) -> System.Collections.Generic.Dictionary[str, str]:
        """
        Customizable dynamic statistics IAlgorithm.runtime_statistics
        
        This codeEntityType is protected.
        """
        ...

    @property
    def state(self) -> System.Collections.Generic.Dictionary[str, str]:
        """
        State of the algorithm
        
        
        This codeEntityType is protected.
        """
        ...

    @state.setter
    def state(self, value: System.Collections.Generic.Dictionary[str, str]) -> None:
        ...

    @property
    def messaging_handler(self) -> QuantConnect.Interfaces.IMessagingHandler:
        """
        The handler responsible for communicating messages to listeners
        
        
        This codeEntityType is protected.
        """
        ...

    @messaging_handler.setter
    def messaging_handler(self, value: QuantConnect.Interfaces.IMessagingHandler) -> None:
        ...

    @property
    def transaction_handler(self) -> QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler:
        """
        The transaction handler used to get the algorithms Orders information
        
        
        This codeEntityType is protected.
        """
        ...

    @transaction_handler.setter
    def transaction_handler(self, value: QuantConnect.Lean.Engine.TransactionHandlers.ITransactionHandler) -> None:
        ...

    @property
    def starting_portfolio_value(self) -> float:
        """
        The algorithms starting portfolio value.
        Used to calculate the portfolio return
        
        
        This codeEntityType is protected.
        """
        ...

    @starting_portfolio_value.setter
    def starting_portfolio_value(self, value: float) -> None:
        ...

    @property
    def algorithm(self) -> QuantConnect.Interfaces.IAlgorithm:
        """
        The algorithm instance
        
        
        This codeEntityType is protected.
        """
        ...

    @algorithm.setter
    def algorithm(self, value: QuantConnect.Interfaces.IAlgorithm) -> None:
        ...

    @property
    def algorithm_currency_symbol(self) -> str:
        """
        Algorithm currency symbol, used in charting
        
        
        This codeEntityType is protected.
        """
        ...

    @algorithm_currency_symbol.setter
    def algorithm_currency_symbol(self, value: str) -> None:
        ...

    @property
    def daily_portfolio_value(self) -> float:
        """
        Closing portfolio value. Used to calculate daily performance.
        
        
        This codeEntityType is protected.
        """
        ...

    @daily_portfolio_value.setter
    def daily_portfolio_value(self, value: float) -> None:
        ...

    @property
    def cumulative_max_portfolio_value(self) -> float:
        """
        Cumulative max portfolio value. Used to calculate drawdown underwater.
        
        
        This codeEntityType is protected.
        """
        ...

    @cumulative_max_portfolio_value.setter
    def cumulative_max_portfolio_value(self, value: float) -> None:
        ...

    @property
    def resample_period(self) -> datetime.timedelta:
        """
        Sampling period for timespans between resamples of the charting equity.
        
        
        This codeEntityType is protected.
        """
        ...

    @resample_period.setter
    def resample_period(self, value: datetime.timedelta) -> None:
        ...

    @property
    def notification_period(self) -> datetime.timedelta:
        """
        How frequently the backtests push messages to the browser.
        
        
        This codeEntityType is protected.
        """
        ...

    @notification_period.setter
    def notification_period(self, value: datetime.timedelta) -> None:
        ...

    @property
    def results_destination_folder(self) -> str:
        """
        Directory location to store results
        
        
        This codeEntityType is protected.
        """
        ...

    @results_destination_folder.setter
    def results_destination_folder(self, value: str) -> None:
        ...

    @property
    def map_file_provider(self) -> QuantConnect.Interfaces.IMapFileProvider:
        """
        The map file provider instance to use
        
        
        This codeEntityType is protected.
        """
        ...

    @map_file_provider.setter
    def map_file_provider(self, value: QuantConnect.Interfaces.IMapFileProvider) -> None:
        ...

    def __init__(self) -> None:
        """
        Creates a new instance
        
        
        This codeEntityType is protected.
        """
        ...

    def add_to_log_store(self, message: str) -> None:
        """
        Save an algorithm message to the log store. Uses a different timestamped method of adding messaging to interweve debug and logging messages.
        
        
        This codeEntityType is protected.
        
        :param message: String message to store
        """
        ...

    def exit(self) -> None:
        """Terminate the result thread and apply any required exit procedures like sending final results"""
        ...

    @overload
    def generate_statistics_results(self, charts: System.Collections.Generic.Dictionary[str, QuantConnect.Chart], profit_loss: System.Collections.Generic.SortedDictionary[datetime.datetime, float] = None, estimated_strategy_capacity: QuantConnect.CapacityEstimate = None) -> QuantConnect.Statistics.StatisticsResults:
        """
        Will generate the statistics results and update the provided runtime statistics
        
        
        This codeEntityType is protected.
        """
        ...

    @overload
    def generate_statistics_results(self, estimated_strategy_capacity: QuantConnect.CapacityEstimate = None) -> QuantConnect.Statistics.StatisticsResults:
        """
        Calculates and gets the current statistics for the algorithm.
        It will use the current charts and profit loss information calculated from the current transaction record
        to generate the results.
        
        
        This codeEntityType is protected.
        
        :returns: The current statistics.
        """
        ...

    def get_algorithm_runtime_statistics(self, summary: System.Collections.Generic.Dictionary[str, str], capacity_estimate: QuantConnect.CapacityEstimate = None) -> System.Collections.Generic.SortedDictionary[str, str]:
        """
        Gets the algorithm runtime statistics
        
        
        This codeEntityType is protected.
        """
        ...

    def get_algorithm_state(self, end_time: typing.Optional[datetime.datetime] = None) -> System.Collections.Generic.Dictionary[str, str]:
        """
        Gets the algorithm state data
        
        
        This codeEntityType is protected.
        """
        ...

    def get_benchmark_value(self, time: typing.Union[datetime.datetime, datetime.date]) -> float:
        """
        Gets the current benchmark value
        
        
        This codeEntityType is protected.
        
        :param time: Time to resolve benchmark value at
        """
        ...

    def get_delta_orders(self, order_events_start_position: int, should_stop: typing.Callable[[int], bool]) -> System.Collections.Generic.Dictionary[int, QuantConnect.Orders.Order]:
        """
        Gets the orders generated starting from the provided ITransactionHandler.order_events position
        
        
        This codeEntityType is protected.
        
        :returns: The delta orders.
        """
        ...

    def get_net_return(self) -> float:
        """
        Gets the algorithm net return
        
        
        This codeEntityType is protected.
        """
        ...

    def get_portfolio_value(self) -> float:
        """
        Gets the current portfolio value
        
        
        This codeEntityType is protected.
        """
        ...

    def get_results_path(self, filename: str) -> str:
        """
        Gets the full path for a results file
        
        
        This codeEntityType is protected.
        
        :param filename: The filename to add to the path
        :returns: The full path, including the filename.
        """
        ...

    def get_server_statistics(self, utc_now: typing.Union[datetime.datetime, datetime.date]) -> System.Collections.Generic.Dictionary[str, str]:
        """
        Gets the current Server statistics
        
        
        This codeEntityType is protected.
        """
        ...

    def initialize(self, parameters: QuantConnect.Lean.Engine.Results.ResultHandlerInitializeParameters) -> None:
        """
        Initialize the result handler with this result packet.
        
        :param parameters: DTO parameters class to initialize a result handler
        """
        ...

    def on_securities_changed(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """Event fired each time that we add/remove securities from the data feed"""
        ...

    def order_event(self, new_event: QuantConnect.Orders.OrderEvent) -> None:
        """
        New order event for the algorithm
        
        :param new_event: New event details
        """
        ...

    def process_algorithm_logs(self, message_queue_limit: typing.Optional[int] = None) -> None:
        """
        Processes algorithm logs.
        Logs of the same type are batched together one per line and are sent out
        
        
        This codeEntityType is protected.
        """
        ...

    def purge_queue(self) -> None:
        """
        Purge/clear any outstanding messages in message queue.
        
        
        This codeEntityType is protected.
        """
        ...

    def run(self) -> None:
        """
        Result handler update method
        
        
        This codeEntityType is protected.
        """
        ...

    @overload
    def sample(self, time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Samples portfolio equity, benchmark, and daily performance
        Called by scheduled event every night at midnight algorithm time
        
        :param time: Current UTC time in the AlgorithmManager loop
        """
        ...

    @overload
    def sample(self, chart_name: str, series_name: str, series_index: int, series_type: QuantConnect.SeriesType, value: QuantConnect.ISeriesPoint, unit: str = "$") -> None:
        """
        Add a sample to the chart specified by the chart_name, and series_name.
        
        
        This codeEntityType is protected.
        
        :param chart_name: String chart name to place the sample.
        :param series_name: Series name for the chart.
        :param series_index: Series chart index - which chart should this series belong
        :param series_type: Series type for the chart.
        :param value: Value for the chart sample.
        :param unit: Unit for the chart axis
        """
        ...

    def sample_benchmark(self, time: typing.Union[datetime.datetime, datetime.date], value: float) -> None:
        """
        Sample the current benchmark performance directly with a time-value pair.
        
        
        This codeEntityType is protected.
        
        :param time: Time of the sample.
        :param value: Current benchmark value.
        """
        ...

    def sample_capacity(self, time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Sample estimated strategy capacity
        
        
        This codeEntityType is protected.
        
        :param time: Time of the sample
        """
        ...

    def sample_drawdown(self, time: typing.Union[datetime.datetime, datetime.date], current_portfolio_value: float) -> None:
        """
        Sample drawdown of equity of the strategy
        
        
        This codeEntityType is protected.
        
        :param time: Time of the sample
        :param current_portfolio_value: Current equity value
        """
        ...

    def sample_equity(self, time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Sample the current equity of the strategy directly with time and using
        the current algorithm equity value in current_algorithm_equity
        
        This codeEntityType is protected.
        
        :param time: Equity candlestick end time
        """
        ...

    def sample_exposure(self, time: typing.Union[datetime.datetime, datetime.date], current_portfolio_value: float) -> None:
        """
        Sample portfolio exposure long/short ratios by security type
        
        
        This codeEntityType is protected.
        
        :param time: Time of the sample
        :param current_portfolio_value: Current value of the portfolio
        """
        ...

    def sample_performance(self, time: typing.Union[datetime.datetime, datetime.date], value: float) -> None:
        """
        Sample the current daily performance directly with a time-value pair.
        
        
        This codeEntityType is protected.
        
        :param time: Time of the sample.
        :param value: Current daily performance value.
        """
        ...

    def sample_portfolio_turnover(self, time: typing.Union[datetime.datetime, datetime.date], current_portfolio_value: float) -> None:
        """
        Sample portfolio turn over of the strategy
        
        
        This codeEntityType is protected.
        
        :param time: Time of the sample
        :param current_portfolio_value: Current equity value
        """
        ...

    def sample_sales_volume(self, time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Sample assets sales volume
        
        
        This codeEntityType is protected.
        
        :param time: Time of the sample
        """
        ...

    def save_logs(self, id: str, logs: typing.List[QuantConnect.Logging.LogEntry]) -> str:
        """
        Returns the location of the logs
        
        :param id: Id that will be incorporated into the algorithm log name
        :param logs: The logs to save
        :returns: The path to the logs.
        """
        ...

    def save_results(self, name: str, result: QuantConnect.Result) -> None:
        """
        Save the results to disk
        
        :param name: The name of the results
        :param result: The results to save
        """
        ...

    def set_algorithm_state(self, error: str, stack: str) -> None:
        """
        Sets the algorithm state data
        
        
        This codeEntityType is protected.
        """
        ...

    def stop_update_runner(self) -> None:
        """
        Stops the update runner task
        
        
        This codeEntityType is protected.
        """
        ...

    def store_insights(self) -> None:
        """
        Save insight results to persistent storage
        
        
        This codeEntityType is protected.
        """
        ...

    def store_order_events(self, utc_time: typing.Union[datetime.datetime, datetime.date], order_events: typing.List[QuantConnect.Orders.OrderEvent]) -> None:
        """
        Stores the order events
        
        
        This codeEntityType is protected.
        
        :param utc_time: The utc date associated with these order events
        :param order_events: The order events to store
        """
        ...

    def store_result(self, packet: QuantConnect.Packets.Packet) -> None:
        """
        Save the snapshot of the total results to storage.
        
        
        This codeEntityType is protected.
        
        :param packet: Packet to store.
        """
        ...

    def summary_statistic(self, name: str, value: str) -> None:
        """
        Sets or updates a custom summary statistic
        
        
        This codeEntityType is protected.
        
        :param name: The statistic name
        :param value: The statistic value
        """
        ...

    def total_trades_count(self) -> int:
        """
        Helper method to get the total trade count statistic
        
        
        This codeEntityType is protected.
        """
        ...

    def update_algorithm_equity(self) -> None:
        """
        Updates the current equity bar with the current equity value from get_portfolio_value
        
        This codeEntityType is protected.
        """
        ...


class IResultHandler(QuantConnect.Statistics.IStatisticsService, metaclass=abc.ABCMeta):
    """
    Handle the results of the backtest: where should we send the profit, portfolio updates:
    Backtester or the Live trading platform:
    """

    @property
    @abc.abstractmethod
    def messages(self) -> System.Collections.Concurrent.ConcurrentQueue[QuantConnect.Packets.Packet]:
        """Put messages to process into the queue so they are processed by this thread."""
        ...

    @messages.setter
    def messages(self, value: System.Collections.Concurrent.ConcurrentQueue[QuantConnect.Packets.Packet]) -> None:
        ...

    @property
    @abc.abstractmethod
    def is_active(self) -> bool:
        """
        Boolean flag indicating the result hander thread is busy.
        False means it has completely finished and ready to dispose.
        """
        ...

    def algorithm_name_updated(self, name: str) -> None:
        """
        Handles updates to the algorithm's name
        
        :param name: The new name
        """
        ...

    def algorithm_tags_updated(self, tags: System.Collections.Generic.HashSet[str]) -> None:
        """
        Handles updates to the algorithm's tags
        
        :param tags: The new tags
        """
        ...

    def brokerage_message(self, brokerage_message_event: QuantConnect.Brokerages.BrokerageMessageEvent) -> None:
        """
        Process brokerage message events
        
        :param brokerage_message_event: The brokerage message event
        """
        ...

    def debug_message(self, message: str) -> None:
        """
        Process debug messages with the preconfigured settings.
        
        :param message: String debug message
        """
        ...

    def error_message(self, error: str, stacktrace: str = ...) -> None:
        """
        Send an error message back to the browser highlighted in red with a stacktrace.
        
        :param error: Error message we'd like shown in console.
        :param stacktrace: Stacktrace information string
        """
        ...

    def exit(self) -> None:
        """Terminate the result thread and apply any required exit procedures like sending final results."""
        ...

    def initialize(self, parameters: QuantConnect.Lean.Engine.Results.ResultHandlerInitializeParameters) -> None:
        """
        Initialize the result handler with this result packet.
        
        :param parameters: DTO parameters class to initialize a result handler
        """
        ...

    def log_message(self, message: str) -> None:
        """
        Send a logging message to the log list for storage.
        
        :param message: Message we'd in the log.
        """
        ...

    def on_securities_changed(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """Event fired each time that we add/remove securities from the data feed"""
        ...

    def order_event(self, new_event: QuantConnect.Orders.OrderEvent) -> None:
        """
        Send a new order event.
        
        :param new_event: Update, processing or cancellation of an order, update the IDE in live mode or ignore in backtesting.
        """
        ...

    def process_synchronous_events(self, force_process: bool = False) -> None:
        """Process any synchronous events in here that are primarily triggered from the algorithm loop"""
        ...

    def runtime_error(self, message: str, stacktrace: str = ...) -> None:
        """
        Send a runtime error message back to the browser highlighted with in red
        
        :param message: Error message.
        :param stacktrace: Stacktrace information string
        """
        ...

    def runtime_statistic(self, key: str, value: str) -> None:
        """
        Set a dynamic runtime statistic to show in the (live) algorithm header
        
        :param key: Runtime headline statistic name
        :param value: Runtime headline statistic value
        """
        ...

    def sample(self, time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Method to update the IResultHandler with various performance metrics.
        Called once a day by scheduled event in AlgorithmManager
        
        :param time: Current time
        """
        ...

    def save_results(self, name: str, result: QuantConnect.Result) -> None:
        """
        Save the results
        
        :param name: The name of the results
        :param result: The results to save
        """
        ...

    def security_type(self, types: typing.List[QuantConnect.SecurityType]) -> None:
        """
        Send a list of security types to the browser
        
        :param types: Security types list inside algorithm
        """
        ...

    def send_status_update(self, status: QuantConnect.AlgorithmStatus, message: str = ...) -> None:
        """
        Send a algorithm status update to the user of the algorithms running state.
        
        :param status: Status enum of the algorithm.
        :param message: Optional string message describing reason for status change.
        """
        ...

    def set_algorithm(self, algorithm: QuantConnect.Interfaces.IAlgorithm, starting_portfolio_value: float) -> None:
        """
        Set the algorithm of the result handler after its been initialized.
        
        :param algorithm: Algorithm object matching IAlgorithm interface
        :param starting_portfolio_value: Algorithm starting capital for statistics calculations
        """
        ...

    def system_debug_message(self, message: str) -> None:
        """
        Process system debug messages with the preconfigured settings.
        
        :param message: String debug message
        """
        ...


class BacktestingResultHandler(QuantConnect.Lean.Engine.Results.BaseResultsHandler, QuantConnect.Lean.Engine.Results.IResultHandler):
    """Backtesting result handler passes messages back from the Lean to the User."""

    @property
    def final_statistics(self) -> System.Collections.Generic.Dictionary[str, str]:
        """A dictionary containing summary statistics"""
        ...

    def __init__(self) -> None:
        """Creates a new instance"""
        ...

    def add_to_log_store(self, message: str) -> None:
        """
        Add message to LogStore
        
        
        This codeEntityType is protected.
        
        :param message: Message to add
        """
        ...

    def algorithm_name_updated(self, name: str) -> None:
        """
        Handles updates to the algorithm's name
        
        :param name: The new name
        """
        ...

    def algorithm_tags_updated(self, tags: System.Collections.Generic.HashSet[str]) -> None:
        """
        Sends a packet communicating an update to the algorithm's tags
        
        :param tags: The new tags
        """
        ...

    def brokerage_message(self, brokerage_message_event: QuantConnect.Brokerages.BrokerageMessageEvent) -> None:
        """
        Process brokerage message events
        
        :param brokerage_message_event: The brokerage message event
        """
        ...

    def configure_console_text_writer(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> None:
        """
        Configures the Console.Out and Console.ErrorTextWriter
        instances. By default, we forward Console.WriteLine(string) to IAlgorithm.debug.
        This is perfect for running in the cloud, but since they're processed asynchronously, the ordering of these
        messages with respect to Log messages is broken. This can lead to differences in regression
        test logs based solely on the ordering of messages. To disable this forwarding, set "forward-console-messages"
        to false in the configuration.
        
        
        This codeEntityType is protected.
        """
        ...

    def debug_message(self, message: str) -> None:
        """
        Send a debug message back to the browser console.
        
        :param message: Message we'd like shown in console.
        """
        ...

    def error_message(self, message: str, stacktrace: str = ...) -> None:
        """
        Send an error message back to the browser highlighted in red with a stacktrace.
        
        :param message: Error message we'd like shown in console.
        :param stacktrace: Stacktrace information string
        """
        ...

    def exit(self) -> None:
        """Terminate the result thread and apply any required exit procedures like sending final results."""
        ...

    def initialize(self, parameters: QuantConnect.Lean.Engine.Results.ResultHandlerInitializeParameters) -> None:
        """Initialize the result handler with this result packet."""
        ...

    def log_message(self, message: str) -> None:
        """
        Send a logging message to the log list for storage.
        
        :param message: Message we'd in the log.
        """
        ...

    def order_event(self, new_event: QuantConnect.Orders.OrderEvent) -> None:
        """
        Handle order event
        
        :param new_event: Event to process
        """
        ...

    def process_synchronous_events(self, force_process: bool = False) -> None:
        """
        Process the synchronous result events, sampling and message reading.
        This method is triggered from the algorithm manager thread.
        """
        ...

    def run(self) -> None:
        """
        The main processing method steps through the messaging queue and processes the messages one by one.
        
        
        This codeEntityType is protected.
        """
        ...

    def runtime_error(self, message: str, stacktrace: str = ...) -> None:
        """
        Send a runtime error message back to the browser highlighted with in red
        
        :param message: Error message.
        :param stacktrace: Stacktrace information string
        """
        ...

    def runtime_statistic(self, key: str, value: str) -> None:
        """
        Set the current runtime statistics of the algorithm.
        These are banner/title statistics which show at the top of the live trading results.
        
        :param key: Runtime headline statistic name
        :param value: Runtime headline statistic value
        """
        ...

    def sample(self, chart_name: str, series_name: str, series_index: int, series_type: QuantConnect.SeriesType, value: QuantConnect.ISeriesPoint, unit: str = "$") -> None:
        """
        Add a sample to the chart specified by the chart_name, and series_name.
        
        
        This codeEntityType is protected.
        
        :param chart_name: String chart name to place the sample.
        :param series_index: Type of chart we should create if it doesn't already exist.
        :param series_name: Series name for the chart.
        :param series_type: Series type for the chart.
        :param value: Value for the chart sample.
        :param unit: Unit of the sample
        """
        ...

    def sample_capacity(self, time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Sample estimated strategy capacity
        
        
        This codeEntityType is protected.
        
        :param time: Time of the sample
        """
        ...

    def sample_range(self, updates: typing.List[QuantConnect.Chart]) -> None:
        """
        Add a range of samples from the users algorithms to the end of our current list.
        
        
        This codeEntityType is protected.
        
        :param updates: Chart updates since the last request.
        """
        ...

    def security_type(self, types: typing.List[QuantConnect.SecurityType]) -> None:
        """Send list of security asset types the algorithm uses to browser."""
        ...

    def send_final_result(self) -> None:
        """
        Send a final analysis result back to the IDE.
        
        
        This codeEntityType is protected.
        """
        ...

    def send_status_update(self, status: QuantConnect.AlgorithmStatus, message: str = ...) -> None:
        """
        Send an algorithm status update to the browser.
        
        :param status: Status enum value.
        :param message: Additional optional status message.
        """
        ...

    def set_algorithm(self, algorithm: QuantConnect.Interfaces.IAlgorithm, starting_portfolio_value: float) -> None:
        """
        Set the Algorithm instance for ths result.
        
        :param algorithm: Algorithm we're working on.
        :param starting_portfolio_value: Algorithm starting capital for statistics calculations
        """
        ...

    def set_summary_statistic(self, name: str, value: str) -> None:
        """
        Sets or updates a custom summary statistic
        
        :param name: The statistic name
        :param value: The statistic value
        """
        ...

    def split_packets(self, delta_charts: System.Collections.Generic.Dictionary[str, QuantConnect.Chart], delta_orders: System.Collections.Generic.Dictionary[int, QuantConnect.Orders.Order], runtime_statistics: System.Collections.Generic.SortedDictionary[str, str], progress: float, server_statistics: System.Collections.Generic.Dictionary[str, str]) -> typing.Iterable[QuantConnect.Packets.BacktestResultPacket]:
        """Run over all the data and break it into smaller packets to ensure they all arrive at the terminal"""
        ...

    def statistics_results(self) -> QuantConnect.Statistics.StatisticsResults:
        """
        Calculates and gets the current statistics for the algorithm
        
        :returns: The current statistics.
        """
        ...

    def store_result(self, packet: QuantConnect.Packets.Packet) -> None:
        """
        Save the snapshot of the total results to storage.
        
        
        This codeEntityType is protected.
        
        :param packet: Packet to store.
        """
        ...

    def system_debug_message(self, message: str) -> None:
        """
        Send a system debug message back to the browser console.
        
        :param message: Message we'd like shown in console.
        """
        ...


class RegressionResultHandler(QuantConnect.Lean.Engine.Results.BacktestingResultHandler):
    """
    Provides a wrapper over the BacktestingResultHandler that logs all order events
    to a separate file
    """

    @property
    def log_file_path(self) -> str:
        """Gets the path used for logging all portfolio changing events, such as orders, TPV, daily holdings values"""
        ...

    @property
    def has_runtime_error(self) -> bool:
        """True if there was a runtime error running the algorithm"""
        ...

    def __init__(self) -> None:
        """Initializes a new instance of the RegressionResultHandler class"""
        ...

    def add_to_log_store(self, message: str) -> None:
        """
        Save an algorithm message to the log store. Uses a different timestamped method of adding messaging to interweve debug and logging messages.
        
        
        This codeEntityType is protected.
        
        :param message: String message to store
        """
        ...

    def configure_console_text_writer(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> None:
        """
        We want to make algorithm messages end up in both the standard regression log file {algorithm}.{language}.log
        as well as the details log {algorithm}.{language}.details.log. The details log is focused on providing a log
        dedicated solely to the algorithm's behavior, void of all QuantConnect.Logging.Log messages
        
        
        This codeEntityType is protected.
        """
        ...

    def debug_message(self, message: str) -> None:
        """
        Send a debug message back to the browser console.
        
        :param message: Message we'd like shown in console.
        """
        ...

    def error_message(self, message: str, stacktrace: str = ...) -> None:
        """
        Send an error message back to the browser highlighted in red with a stacktrace.
        
        :param message: Error message we'd like shown in console.
        :param stacktrace: Stacktrace information string
        """
        ...

    def exit(self) -> None:
        """
        Terminate the result thread and apply any required exit procedures.
        Save orders log files to disk.
        """
        ...

    def log_message(self, message: str) -> None:
        """
        Send a logging message to the log list for storage.
        
        :param message: Message we'd in the log.
        """
        ...

    def on_securities_changed(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """Event fired each time that we add/remove securities from the data feed"""
        ...

    def order_event(self, new_event: QuantConnect.Orders.OrderEvent) -> None:
        """
        Log the order and order event to the dedicated log file for this regression algorithm
        
        :param new_event: New order event details
        """
        ...

    def process_synchronous_events(self, force_process: bool = False) -> None:
        """
        Runs at the end of each time loop. When HighFidelityLogging is enabled, we'll
        log each piece of data to allow for faster determination of regression causes
        """
        ...

    def runtime_error(self, message: str, stacktrace: str = ...) -> None:
        """
        Send a runtime error message back to the browser highlighted with in red
        
        :param message: Error message.
        :param stacktrace: Stacktrace information string
        """
        ...

    def runtime_statistic(self, key: str, value: str) -> None:
        """
        Set the current runtime statistics of the algorithm.
        These are banner/title statistics which show at the top of the live trading results.
        
        :param key: Runtime headline statistic name
        :param value: Runtime headline statistic value
        """
        ...

    def sample_performance(self, time: typing.Union[datetime.datetime, datetime.date], value: float) -> None:
        """
        Runs on date changes, use this to log TPV and holdings values each day
        
        
        This codeEntityType is protected.
        """
        ...

    def save_results(self, name: str, result: QuantConnect.Result) -> None:
        """Save the results to disk"""
        ...

    def security_type(self, types: typing.List[QuantConnect.SecurityType]) -> None:
        """Send list of security asset types the algortihm uses to browser."""
        ...

    def set_algorithm(self, algorithm: QuantConnect.Interfaces.IAlgorithm, starting_portfolio_value: float) -> None:
        """Initializes the stream writer using the algorithm's id (name) in the file path"""
        ...

    def system_debug_message(self, message: str) -> None:
        """
        Send a system debug message back to the browser console.
        
        :param message: Message we'd like shown in console.
        """
        ...


class LiveTradingResultHandler(QuantConnect.Lean.Engine.Results.BaseResultsHandler, QuantConnect.Lean.Engine.Results.IResultHandler):
    """Live trading result handler implementation passes the messages to the QC live trading interface."""

    def __init__(self) -> None:
        """Creates a new instance"""
        ...

    def add_to_log_store(self, message: str) -> None:
        """
        Save an algorithm message to the log store. Uses a different timestamped method of adding messaging to interweve debug and logging messages.
        
        
        This codeEntityType is protected.
        
        :param message: String message to send to browser.
        """
        ...

    def algorithm_name_updated(self, name: str) -> None:
        """
        Handles updates to the algorithm's name
        
        :param name: The new name
        """
        ...

    def algorithm_tags_updated(self, tags: System.Collections.Generic.HashSet[str]) -> None:
        """
        Handles updates to the algorithm's tags
        
        :param tags: The new tags
        """
        ...

    def brokerage_message(self, brokerage_message_event: QuantConnect.Brokerages.BrokerageMessageEvent) -> None:
        """
        Process brokerage message events
        
        :param brokerage_message_event: The brokerage message event
        """
        ...

    def create_safe_chart_name(self, chart_name: str) -> str:
        """
        Escape the chartname so that it can be saved to a file system
        
        
        This codeEntityType is protected.
        
        :param chart_name: The name of a chart
        :returns: The name of the chart will all escape all characters except RFC 2396 unreserved characters.
        """
        ...

    def debug_message(self, message: str) -> None:
        """
        Send a live trading debug message to the live console.
        
        :param message: Message we'd like shown in console.
        """
        ...

    def error_message(self, message: str, stacktrace: str = ...) -> None:
        """
        Send an error message back to the browser console and highlight it read.
        
        :param message: Message we'd like shown in console.
        :param stacktrace: Stacktrace to show in the console.
        """
        ...

    def exit(self) -> None:
        """Terminate the result thread and apply any required exit procedures like sending final results"""
        ...

    def get_benchmark_value(self, time: typing.Union[datetime.datetime, datetime.date]) -> float:
        """
        Gets the current benchmark value
        
        
        This codeEntityType is protected.
        
        :param time: Time to resolve benchmark value at
        """
        ...

    @staticmethod
    def get_holdings(securities: typing.List[QuantConnect.Securities.Security], subscription_data_config_service: QuantConnect.Interfaces.ISubscriptionDataConfigService, only_invested: bool = False) -> System.Collections.Generic.Dictionary[str, QuantConnect.Holding]:
        """Helper method to fetch the algorithm holdings"""
        ...

    def get_portfolio_value(self) -> float:
        """
        Gets the current portfolio value
        
        
        This codeEntityType is protected.
        """
        ...

    def initialize(self, parameters: QuantConnect.Lean.Engine.Results.ResultHandlerInitializeParameters) -> None:
        """
        Initialize the result handler with this result packet.
        
        :param parameters: DTO parameters class to initialize a result handler
        """
        ...

    def log_message(self, message: str) -> None:
        """
        Log string messages and send them to the console.
        
        :param message: String message wed like logged.
        """
        ...

    def on_securities_changed(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """
        Event fired each time that we add/remove securities from the data feed.
        On Security change we re determine when should we sample charts, if the user added Crypto, Forex or an extended market hours subscription
        we will always sample charts. Else, we will keep the exchange per market to query later on demand
        """
        ...

    def order_event(self, new_event: QuantConnect.Orders.OrderEvent) -> None:
        """
        New order event for the algorithm
        
        :param new_event: New event details
        """
        ...

    def process_synchronous_events(self, force_process: bool = False) -> None:
        """
        Process the synchronous result events, sampling and message reading.
        This method is triggered from the algorithm manager thread.
        """
        ...

    def run(self) -> None:
        """
        Live trading result handler thread.
        
        
        This codeEntityType is protected.
        """
        ...

    def runtime_error(self, message: str, stacktrace: str = ...) -> None:
        """
        Send a runtime error back to the users browser and highlight it red.
        
        :param message: Runtime error message
        :param stacktrace: Associated error stack trace.
        """
        ...

    def runtime_statistic(self, key: str, value: str) -> None:
        """
        Set a dynamic runtime statistic to show in the (live) algorithm header
        
        :param key: Runtime headline statistic name
        :param value: Runtime headline statistic value
        """
        ...

    @overload
    def sample(self, chart_name: str, series_name: str, series_index: int, series_type: QuantConnect.SeriesType, value: QuantConnect.ISeriesPoint, unit: str = "$") -> None:
        """
        Add a sample to the chart specified by the chart_name, and series_name.
        
        
        This codeEntityType is protected.
        
        :param chart_name: String chart name to place the sample.
        :param series_name: Series name for the chart.
        :param series_index: Series chart index - which chart should this series belong
        :param series_type: Series type for the chart.
        :param value: Value for the chart sample.
        :param unit: Unit for the chart axis
        """
        ...

    @overload
    def sample(self, time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Samples portfolio equity, benchmark, and daily performance
        
        :param time: Current UTC time in the AlgorithmManager loop
        """
        ...

    def sample_range(self, updates: typing.List[QuantConnect.Chart]) -> None:
        """
        Add a range of samples from the users algorithms to the end of our current list.
        
        
        This codeEntityType is protected.
        
        :param updates: Chart updates since the last request.
        """
        ...

    def save_logs(self, id: str, logs: typing.List[QuantConnect.Logging.LogEntry]) -> str:
        """
        Process the log entries and save it to permanent storage
        
        :param id: Id that will be incorporated into the algorithm log name
        :param logs: Log list
        :returns: Returns the location of the logs.
        """
        ...

    def security_type(self, types: typing.List[QuantConnect.SecurityType]) -> None:
        """
        Send a list of secutity types that the algorithm trades to the browser to show the market clock - is this market open or closed!
        
        :param types: List of security types
        """
        ...

    def send_final_result(self) -> None:
        """
        Send a final analysis result back to the IDE.
        
        
        This codeEntityType is protected.
        """
        ...

    def send_status_update(self, status: QuantConnect.AlgorithmStatus, message: str = ...) -> None:
        """
        Send a algorithm status update to the user of the algorithms running state.
        
        :param status: Status enum of the algorithm.
        :param message: Optional string message describing reason for status change.
        """
        ...

    def set_algorithm(self, algorithm: QuantConnect.Interfaces.IAlgorithm, starting_portfolio_value: float) -> None:
        """
        Set the algorithm of the result handler after its been initialized.
        
        :param algorithm: Algorithm object matching IAlgorithm interface
        :param starting_portfolio_value: Algorithm starting capital for statistics calculations
        """
        ...

    def set_next_status_update(self) -> None:
        """
        Assigns the next earliest status update time
        
        
        This codeEntityType is protected.
        """
        ...

    def set_summary_statistic(self, name: str, value: str) -> None:
        """
        Sets or updates a custom summary statistic
        
        :param name: The statistic name
        :param value: The statistic value
        """
        ...

    def statistics_results(self) -> QuantConnect.Statistics.StatisticsResults:
        """
        Calculates and gets the current statistics for the algorithm
        
        :returns: The current statistics.
        """
        ...

    def store_order_events(self, utc_time: typing.Union[datetime.datetime, datetime.date], order_events: typing.List[QuantConnect.Orders.OrderEvent]) -> None:
        """
        Stores the order events
        
        
        This codeEntityType is protected.
        
        :param utc_time: The utc date associated with these order events
        :param order_events: The order events to store
        """
        ...

    def store_result(self, packet: QuantConnect.Packets.Packet) -> None:
        """
        Save the snapshot of the total results to storage.
        
        
        This codeEntityType is protected.
        
        :param packet: Packet to store.
        """
        ...

    def system_debug_message(self, message: str) -> None:
        """
        Send a live trading system debug message to the live console.
        
        :param message: Message we'd like shown in console.
        """
        ...


class BacktestProgressMonitor(System.Object):
    """Monitors and reports the progress of a backtest"""

    @property
    def total_days(self) -> int:
        """Gets the total days the algorithm will run"""
        ...

    @property
    def processed_days(self) -> int:
        """Gets the current days the algorithm has been running for"""
        ...

    @property
    def progress(self) -> float:
        """Gets the current progress of the backtest"""
        ...

    def __init__(self, time_keeper: QuantConnect.Interfaces.ITimeKeeper, end_utc_time: typing.Union[datetime.datetime, datetime.date]) -> None:
        """
        Creates a new instance
        
        :param time_keeper: The time keeper to use
        :param end_utc_time: The end UTC time
        """
        ...

    def invalidate_processed_days(self) -> None:
        """Invalidates the processed days count value so it gets recalculated next time it is needed"""
        ...





from typing import overload
from enum import IntEnum
import abc
import typing

import QuantConnect.Notifications
import System
import System.Collections.Concurrent
import System.Collections.Generic

JsonConverter = typing.Any


class Notification(System.Object, metaclass=abc.ABCMeta):
    """Local/desktop implementation of messaging system for Lean Engine."""

    def send(self) -> None:
        """Method for sending implementations of notification object types."""
        ...


class NotificationWeb(QuantConnect.Notifications.Notification):
    """Web Notification Class"""

    @property
    def headers(self) -> System.Collections.Generic.Dictionary[str, str]:
        """Optional email headers"""
        ...

    @headers.setter
    def headers(self, value: System.Collections.Generic.Dictionary[str, str]) -> None:
        ...

    @property
    def address(self) -> str:
        """Send a notification message to this web address"""
        ...

    @address.setter
    def address(self, value: str) -> None:
        ...

    @property
    def data(self) -> System.Object:
        """Object data to send."""
        ...

    @data.setter
    def data(self, value: System.Object) -> None:
        ...

    def __init__(self, address: str, data: typing.Any = None, headers: System.Collections.Generic.Dictionary[str, str] = None) -> None:
        """
        Constructor for sending a notification SMS to a specified phone number
        
        :param address: Address to send to
        :param data: Data to send
        :param headers: Optional headers to use
        """
        ...


class NotificationSms(QuantConnect.Notifications.Notification):
    """Sms Notification Class"""

    @property
    def phone_number(self) -> str:
        """Send a notification message to this phone number"""
        ...

    @phone_number.setter
    def phone_number(self, value: str) -> None:
        ...

    @property
    def message(self) -> str:
        """Message to send. Limited to 160 characters"""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    def __init__(self, number: str, message: str) -> None:
        """
        Constructor for sending a notification SMS to a specified phone number
        
        :param number: 
        :param message: 
        """
        ...


class NotificationEmail(QuantConnect.Notifications.Notification):
    """Email notification data."""

    @property
    def headers(self) -> System.Collections.Generic.Dictionary[str, str]:
        """Optional email headers"""
        ...

    @headers.setter
    def headers(self, value: System.Collections.Generic.Dictionary[str, str]) -> None:
        ...

    @property
    def address(self) -> str:
        """Send to address:"""
        ...

    @address.setter
    def address(self, value: str) -> None:
        ...

    @property
    def subject(self) -> str:
        """Email subject"""
        ...

    @subject.setter
    def subject(self, value: str) -> None:
        ...

    @property
    def message(self) -> str:
        """Message to send."""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    @property
    def data(self) -> str:
        """Email Data"""
        ...

    @data.setter
    def data(self, value: str) -> None:
        ...

    def __init__(self, address: str, subject: str = ..., message: str = ..., data: str = ..., headers: System.Collections.Generic.Dictionary[str, str] = None) -> None:
        """
        Default constructor for sending an email notification
        
        :param address: Address to send to, if null will default to users email. Will throw ArgumentException if invalid
        Validate.EmailAddress
        :param subject: Subject of the email. Will set to string.Empty if null
        :param message: Message body of the email. Will set to string.Empty if null
        :param data: Data to attach to the email. Will set to string.Empty if null
        :param headers: Optional email headers to use
        """
        ...


class NotificationTelegram(QuantConnect.Notifications.Notification):
    """Telegram notification data"""

    @property
    def id(self) -> str:
        """
        Send a notification message to this user on Telegram
        Can be either a personal ID or Group ID.
        """
        ...

    @id.setter
    def id(self, value: str) -> None:
        ...

    @property
    def message(self) -> str:
        """Message to send. Limited to 4096 characters"""
        ...

    @message.setter
    def message(self, value: str) -> None:
        ...

    @property
    def token(self) -> str:
        """Token to use"""
        ...

    @token.setter
    def token(self, value: str) -> None:
        ...

    def __init__(self, id: str, message: str, token: str = None) -> None:
        """
        Constructor for sending a telegram notification to a specific User ID
        or group ID. Note: The bot must have an open chat with the user or be
        added to the group for messages to deliver.
        
        :param id: User Id or Group Id to send the message too
        :param message: Message to send
        :param token: Bot token to use, if null defaults to "telegram-token"
        in config on send
        """
        ...


class NotificationFtp(QuantConnect.Notifications.Notification):
    """FTP notification data"""

    @property
    def secure(self) -> bool:
        """Whether to use SFTP or FTP."""
        ...

    @property
    def hostname(self) -> str:
        """The FTP server hostname."""
        ...

    @property
    def port(self) -> int:
        """The FTP server port."""
        ...

    @property
    def username(self) -> str:
        """The FTP server username."""
        ...

    @property
    def password(self) -> str:
        """The FTP server password."""
        ...

    @property
    def file_path(self) -> str:
        """The path to file on the FTP server."""
        ...

    @property
    def file_content(self) -> str:
        """The contents of the file to send."""
        ...

    @property
    def private_key(self) -> str:
        """The private key to use for authentication (optional)."""
        ...

    @property
    def private_key_passphrase(self) -> str:
        """The passphrase for the private key (optional)."""
        ...

    @overload
    def __init__(self, hostname: str, username: str, password: str, file_path: str, file_content: typing.List[int], secure: bool = True, port: typing.Optional[int] = None) -> None:
        """
        Constructor for a notification to sent as a file to an FTP server using password authentication.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param password: The FTP server password
        :param file_path: The path to file on the FTP server
        :param file_content: The contents of the file
        :param secure: Whether to use SFTP or FTP. Defaults to true
        :param port: The FTP server port. Defaults to 21
        """
        ...

    @overload
    def __init__(self, hostname: str, username: str, private_key: str, private_key_passphrase: str, file_path: str, file_content: typing.List[int], port: typing.Optional[int] = None) -> None:
        """
        Constructor for a notification to sent as a file to an FTP server over SFTP using SSH keys.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param private_key: The private SSH key to use for authentication
        :param private_key_passphrase: The optional passphrase to decrypt the private key.
        This can be empty or null if the private key is not encrypted
        :param file_path: The path to file on the FTP server
        :param file_content: The contents of the file
        :param port: The FTP server port. Defaults to 21
        """
        ...

    @overload
    def __init__(self, hostname: str, username: str, password: str, file_path: str, file_content: str, secure: bool = True, port: typing.Optional[int] = None) -> None:
        """
        Constructor for a notification to sent as a file to an FTP server using password authentication.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param password: The FTP server password
        :param file_path: The path to file on the FTP server
        :param file_content: The contents of the file
        :param secure: Whether to use SFTP or FTP. Defaults to true
        :param port: The FTP server port. Defaults to 21
        """
        ...

    @overload
    def __init__(self, hostname: str, username: str, private_key: str, private_key_passphrase: str, file_path: str, file_content: str, port: typing.Optional[int] = None) -> None:
        """
        Constructor for a notification to sent as a file to an FTP server over SFTP using SSH keys.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param private_key: The private SSH key to use for authentication
        :param private_key_passphrase: The optional passphrase to decrypt the private key.
        This can be empty or null if the private key is not encrypted
        :param file_path: The path to file on the FTP server
        :param file_content: The contents of the file
        :param port: The FTP server port. Defaults to 21
        """
        ...


class NotificationExtensions(System.Object):
    """Extension methods for Notification"""

    @staticmethod
    def can_send(notification: QuantConnect.Notifications.Notification) -> bool:
        """
        Check if the notification can be sent (implements the Notification.send method)
        
        :param notification: The notification
        :returns: Whether the notification can be sent.
        """
        ...


class NotificationJsonConverter(JsonConverter):
    """Defines a JsonConverter to be used when deserializing to the Notification class."""

    @property
    def can_write(self) -> bool:
        """Use default implementation to write the json"""
        ...

    def can_convert(self, object_type: typing.Type) -> bool:
        """
        Determines whether this instance can convert the specified object type.
        
        :param object_type: Type of the object.
        :returns: true if this instance can convert the specified object type; otherwise, false.
        """
        ...

    def read_json(self, reader: typing.Any, object_type: typing.Type, existing_value: typing.Any, serializer: typing.Any) -> System.Object:
        """
        Reads the JSON representation of the object.
        
        :param reader: The Newtonsoft.Json.JsonReader to read from.
        :param object_type: Type of the object.
        :param existing_value: The existing value of object being read.
        :param serializer: The calling serializer.
        :returns: The object value.
        """
        ...

    def write_json(self, writer: typing.Any, value: typing.Any, serializer: typing.Any) -> None:
        """
        Writes the JSON representation of the object.
        
        :param writer: The Newtonsoft.Json.JsonWriter to write to.
        :param value: The value.
        :param serializer: The calling serializer.
        """
        ...


class NotificationManager(System.Object):
    """Local/desktop implementation of messaging system for Lean Engine."""

    @property
    def messages(self) -> System.Collections.Concurrent.ConcurrentQueue[QuantConnect.Notifications.Notification]:
        """Public access to the messages"""
        ...

    @messages.setter
    def messages(self, value: System.Collections.Concurrent.ConcurrentQueue[QuantConnect.Notifications.Notification]) -> None:
        ...

    def __init__(self, live_mode: bool) -> None:
        """Initialize the messaging system"""
        ...

    @overload
    def email(self, address: str, subject: str, message: str, data: str, headers: typing.Any) -> bool:
        """
        Send an email to the address specified for live trading notifications.
        
        :param subject: Subject of the email
        :param message: Message body, up to 10kb
        :param data: Data attachment (optional)
        :param address: Email address to send to, if null will default to users email
        :param headers: Optional email headers to use
        """
        ...

    @overload
    def email(self, address: str, subject: str, message: str, data: str = ..., headers: System.Collections.Generic.Dictionary[str, str] = None) -> bool:
        """
        Send an email to the address specified for live trading notifications.
        
        :param subject: Subject of the email
        :param message: Message body, up to 10kb
        :param data: Data attachment (optional)
        :param address: Email address to send to, if null will default to users email
        :param headers: Optional email headers to use
        """
        ...

    @overload
    def ftp(self, hostname: str, username: str, password: str, file_path: str, file_content: typing.List[int], port: typing.Optional[int] = None) -> bool:
        """
        Send a file to the FTP specified server using password authentication over unsecure FTP.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param password: The FTP server password
        :param file_path: The path to file on the FTP server
        :param file_content: The contents of the file
        :param port: The FTP server port. Defaults to 21
        """
        ...

    @overload
    def ftp(self, hostname: str, username: str, password: str, file_path: str, file_content: str, port: typing.Optional[int] = None) -> bool:
        """
        Send a file to the FTP specified server using password authentication over unsecure FTP.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param password: The FTP server password
        :param file_path: The path to file on the FTP server
        :param file_content: The string contents of the file
        :param port: The FTP server port. Defaults to 21
        """
        ...

    @overload
    def sftp(self, hostname: str, username: str, password: str, file_path: str, file_content: typing.List[int], port: typing.Optional[int] = None) -> bool:
        """
        Send a file to the FTP specified server using password authentication over SFTP.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param password: The FTP server password
        :param file_path: The path to file on the FTP server
        :param file_content: The contents of the file
        :param port: The FTP server port. Defaults to 21
        """
        ...

    @overload
    def sftp(self, hostname: str, username: str, password: str, file_path: str, file_content: str, port: typing.Optional[int] = None) -> bool:
        """
        Send a file to the FTP specified server using password authentication over SFTP.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param password: The FTP server password
        :param file_path: The path to file on the FTP server
        :param file_content: The string contents of the file
        :param port: The FTP server port. Defaults to 21
        """
        ...

    @overload
    def sftp(self, hostname: str, username: str, private_key: str, private_key_passphrase: str, file_path: str, file_content: typing.List[int], port: typing.Optional[int] = None) -> bool:
        """
        Send a file to the FTP specified server using password authentication over SFTP using SSH keys.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param private_key: The private SSH key to use for authentication
        :param private_key_passphrase: The optional passphrase to decrypt the private key.
        This can be empty or null if the private key is not encrypted
        :param file_path: The path to file on the FTP server
        :param file_content: The contents of the file
        :param port: The FTP server port. Defaults to 21
        """
        ...

    @overload
    def sftp(self, hostname: str, username: str, private_key: str, private_key_passphrase: str, file_path: str, file_content: str, port: typing.Optional[int] = None) -> bool:
        """
        Send a file to the FTP specified server using password authentication over SFTP using SSH keys.
        
        :param hostname: FTP server hostname
        :param username: The FTP server username
        :param private_key: The private SSH key to use for authentication
        :param private_key_passphrase: The optional passphrase to decrypt the private key.
        This can be empty or null if the private key is not encrypted
        :param file_path: The path to file on the FTP server
        :param file_content: The string contents of the file
        :param port: The FTP server port. Defaults to 21
        """
        ...

    def sms(self, phone_number: str, message: str) -> bool:
        """
        Send an SMS to the phone number specified
        
        :param phone_number: Phone number to send to
        :param message: Message to send
        """
        ...

    def telegram(self, id: str, message: str, token: str = None) -> bool:
        """
        Send a telegram message to the chat ID specified, supply token for custom bot.
        Note: Requires bot to have chat with user or be in the group specified by ID.
        
        :param id: Chat or group ID to send message to
        :param message: Message to send
        :param token: Bot token to use for this message
        """
        ...

    @overload
    def web(self, address: str, data: typing.Any, headers: typing.Any) -> bool:
        """
        Place REST POST call to the specified address with the specified DATA.
        Python overload for Headers parameter.
        
        :param address: Endpoint address
        :param data: Data to send in body JSON encoded
        :param headers: Optional headers to use
        """
        ...

    @overload
    def web(self, address: str, data: typing.Any = None, headers: System.Collections.Generic.Dictionary[str, str] = None) -> bool:
        """
        Place REST POST call to the specified address with the specified DATA.
        
        :param address: Endpoint address
        :param data: Data to send in body JSON encoded (optional)
        :param headers: Optional headers to use
        """
        ...





from typing import overload
from enum import IntEnum
import typing

import System
import System.ComponentModel.DataAnnotations.Schema


class InversePropertyAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def property(self) -> str:
        ...

    def __init__(self, property: str) -> None:
        ...


class ForeignKeyAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def name(self) -> str:
        ...

    def __init__(self, name: str) -> None:
        ...


class ComplexTypeAttribute(System.Attribute):
    """This class has no documentation."""


class DatabaseGeneratedOption(IntEnum):
    """This class has no documentation."""

    NONE = 0

    IDENTITY = 1

    COMPUTED = 2


class DatabaseGeneratedAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def database_generated_option(self) -> System.ComponentModel.DataAnnotations.Schema.DatabaseGeneratedOption:
        ...

    def __init__(self, database_generated_option: System.ComponentModel.DataAnnotations.Schema.DatabaseGeneratedOption) -> None:
        ...


class ColumnAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def name(self) -> str:
        ...

    @property
    def order(self) -> int:
        ...

    @order.setter
    def order(self, value: int) -> None:
        ...

    @property
    def type_name(self) -> str:
        ...

    @type_name.setter
    def type_name(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, name: str) -> None:
        ...


class NotMappedAttribute(System.Attribute):
    """This class has no documentation."""


class TableAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def name(self) -> str:
        ...

    @property
    def schema(self) -> str:
        ...

    @schema.setter
    def schema(self, value: str) -> None:
        ...

    def __init__(self, name: str) -> None:
        ...





from typing import overload
from enum import IntEnum
import abc

import QuantConnect.Data.UniverseSelection
import QuantConnect.Interfaces
import QuantConnect.Lean.Engine
import QuantConnect.Lean.Engine.Server
import QuantConnect.Packets
import System


class ILeanManager(System.IDisposable, metaclass=abc.ABCMeta):
    """Provides scope into Lean that is convenient for managing a lean instance"""

    def initialize(self, system_handlers: QuantConnect.Lean.Engine.LeanEngineSystemHandlers, algorithm_handlers: QuantConnect.Lean.Engine.LeanEngineAlgorithmHandlers, job: QuantConnect.Packets.AlgorithmNodePacket, algorithm_manager: QuantConnect.Lean.Engine.AlgorithmManager) -> None:
        """
        Initialize the ILeanManager implementation
        
        :param system_handlers: Exposes lean engine system handlers running LEAN
        :param algorithm_handlers: Exposes the lean algorithm handlers running lean
        :param job: The job packet representing either a live or backtest Lean instance
        :param algorithm_manager: The Algorithm manager
        """
        ...

    def on_algorithm_end(self) -> None:
        """This method is called before algorithm termination"""
        ...

    def on_algorithm_start(self) -> None:
        """This method is called after algorithm initialization"""
        ...

    def on_securities_changed(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """Callback fired each time that we add/remove securities from the data feed"""
        ...

    def set_algorithm(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> None:
        """
        Sets the IAlgorithm instance in the ILeanManager
        
        :param algorithm: The IAlgorithm instance being run
        """
        ...

    def update(self) -> None:
        """Update ILeanManager with the IAlgorithm instance"""
        ...


class LocalLeanManager(System.Object, QuantConnect.Lean.Engine.Server.ILeanManager):
    """NOP implementation of the ILeanManager interface"""

    @property
    def algorithm(self) -> QuantConnect.Interfaces.IAlgorithm:
        """
        The current algorithm
        
        
        This codeEntityType is protected.
        """
        ...

    @algorithm.setter
    def algorithm(self, value: QuantConnect.Interfaces.IAlgorithm) -> None:
        ...

    @property
    def system_handlers(self) -> QuantConnect.Lean.Engine.LeanEngineSystemHandlers:
        """
        The system handlers
        
        
        This codeEntityType is protected.
        """
        ...

    @system_handlers.setter
    def system_handlers(self, value: QuantConnect.Lean.Engine.LeanEngineSystemHandlers) -> None:
        ...

    @property
    def algorithm_handlers(self) -> QuantConnect.Lean.Engine.LeanEngineAlgorithmHandlers:
        """
        The algorithm handlers
        
        
        This codeEntityType is protected.
        """
        ...

    @algorithm_handlers.setter
    def algorithm_handlers(self, value: QuantConnect.Lean.Engine.LeanEngineAlgorithmHandlers) -> None:
        ...

    def dispose(self) -> None:
        """Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources."""
        ...

    def initialize(self, system_handlers: QuantConnect.Lean.Engine.LeanEngineSystemHandlers, algorithm_handlers: QuantConnect.Lean.Engine.LeanEngineAlgorithmHandlers, job: QuantConnect.Packets.AlgorithmNodePacket, algorithm_manager: QuantConnect.Lean.Engine.AlgorithmManager) -> None:
        """
        Empty implementation of the ILeanManager interface
        
        :param system_handlers: Exposes lean engine system handlers running LEAN
        :param algorithm_handlers: Exposes the lean algorithm handlers running lean
        :param job: The job packet representing either a live or backtest Lean instance
        :param algorithm_manager: The Algorithm manager
        """
        ...

    def on_algorithm_end(self) -> None:
        """This method is called before algorithm termination"""
        ...

    def on_algorithm_start(self) -> None:
        """This method is called after algorithm initialization"""
        ...

    def on_securities_changed(self, changes: QuantConnect.Data.UniverseSelection.SecurityChanges) -> None:
        """Callback fired each time that we add/remove securities from the data feed"""
        ...

    def set_algorithm(self, algorithm: QuantConnect.Interfaces.IAlgorithm) -> None:
        """
        Sets the IAlgorithm instance in the ILeanManager
        
        :param algorithm: The IAlgorithm instance being run
        """
        ...

    def set_command_handler(self) -> None:
        """
        Set the command handler to use, protected for testing purposes
        
        
        This codeEntityType is protected.
        """
        ...

    def update(self) -> None:
        """Execute the commands using the IAlgorithm instance"""
        ...





from typing import overload
from enum import IntEnum
import abc
import datetime
import typing
import warnings

import System
import System.Collections
import System.Collections.Generic
import System.IO
import System.Net
import System.Net.Sockets
import System.Runtime.Serialization
import System.Security
import System.Security.Authentication.ExtendedProtection

System_Net_IPNetwork = typing.Any
System_Net_IPAddress = typing.Any
System_Net_IPEndPoint = typing.Any


class WebUtility(System.Object):
    """This class has no documentation."""

    @staticmethod
    @overload
    def html_decode(value: str) -> str:
        ...

    @staticmethod
    @overload
    def html_decode(value: str, output: System.IO.TextWriter) -> None:
        ...

    @staticmethod
    @overload
    def html_encode(value: str) -> str:
        ...

    @staticmethod
    @overload
    def html_encode(value: str, output: System.IO.TextWriter) -> None:
        ...

    @staticmethod
    def url_decode(encoded_value: str) -> str:
        ...

    @staticmethod
    def url_decode_to_bytes(encoded_value: typing.List[int], offset: int, count: int) -> typing.List[int]:
        ...

    @staticmethod
    def url_encode(value: str) -> str:
        ...

    @staticmethod
    def url_encode_to_bytes(value: typing.List[int], offset: int, count: int) -> typing.List[int]:
        ...


class ICredentials(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def get_credential(self, uri: System.Uri, auth_type: str) -> System.Net.NetworkCredential:
        ...


class ICredentialsByHost(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def get_credential(self, host: str, port: int, authentication_type: str) -> System.Net.NetworkCredential:
        ...


class NetworkCredential(System.Object, System.Net.ICredentials, System.Net.ICredentialsByHost):
    """This class has no documentation."""

    @property
    def user_name(self) -> str:
        ...

    @user_name.setter
    def user_name(self, value: str) -> None:
        ...

    @property
    def password(self) -> str:
        ...

    @password.setter
    def password(self, value: str) -> None:
        ...

    @property
    def secure_password(self) -> System.Security.SecureString:
        ...

    @secure_password.setter
    def secure_password(self, value: System.Security.SecureString) -> None:
        ...

    @property
    def domain(self) -> str:
        ...

    @domain.setter
    def domain(self, value: str) -> None:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, user_name: str, password: str) -> None:
        ...

    @overload
    def __init__(self, user_name: str, password: str, domain: str) -> None:
        ...

    @overload
    def __init__(self, user_name: str, password: System.Security.SecureString) -> None:
        ...

    @overload
    def __init__(self, user_name: str, password: System.Security.SecureString, domain: str) -> None:
        ...

    @overload
    def get_credential(self, uri: System.Uri, authentication_type: str) -> System.Net.NetworkCredential:
        ...

    @overload
    def get_credential(self, host: str, port: int, authentication_type: str) -> System.Net.NetworkCredential:
        ...


class DecompressionMethods(IntEnum):
    """This class has no documentation."""

    NONE = 0

    G_ZIP = ...

    DEFLATE = ...

    BROTLI = ...

    ALL = ...


class IWebProxy(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    @abc.abstractmethod
    def credentials(self) -> System.Net.ICredentials:
        ...

    @credentials.setter
    def credentials(self, value: System.Net.ICredentials) -> None:
        ...

    def get_proxy(self, destination: System.Uri) -> System.Uri:
        ...

    def is_bypassed(self, host: System.Uri) -> bool:
        ...


class TransportContext(System.Object, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def get_channel_binding(self, kind: System.Security.Authentication.ExtendedProtection.ChannelBindingKind) -> System.Security.Authentication.ExtendedProtection.ChannelBinding:
        ...


class Cookie(System.Object):
    """This class has no documentation."""

    @property
    def comment(self) -> str:
        ...

    @comment.setter
    def comment(self, value: str) -> None:
        ...

    @property
    def comment_uri(self) -> System.Uri:
        ...

    @comment_uri.setter
    def comment_uri(self, value: System.Uri) -> None:
        ...

    @property
    def http_only(self) -> bool:
        ...

    @http_only.setter
    def http_only(self, value: bool) -> None:
        ...

    @property
    def discard(self) -> bool:
        ...

    @discard.setter
    def discard(self, value: bool) -> None:
        ...

    @property
    def domain(self) -> str:
        ...

    @domain.setter
    def domain(self, value: str) -> None:
        ...

    @property
    def expired(self) -> bool:
        ...

    @expired.setter
    def expired(self, value: bool) -> None:
        ...

    @property
    def expires(self) -> datetime.datetime:
        ...

    @expires.setter
    def expires(self, value: datetime.datetime) -> None:
        ...

    @property
    def name(self) -> str:
        ...

    @name.setter
    def name(self, value: str) -> None:
        ...

    @property
    def path(self) -> str:
        ...

    @path.setter
    def path(self, value: str) -> None:
        ...

    @property
    def port(self) -> str:
        ...

    @port.setter
    def port(self, value: str) -> None:
        ...

    @property
    def secure(self) -> bool:
        ...

    @secure.setter
    def secure(self, value: bool) -> None:
        ...

    @property
    def time_stamp(self) -> datetime.datetime:
        ...

    @property
    def value(self) -> str:
        ...

    @value.setter
    def value(self, value: str) -> None:
        ...

    @property
    def version(self) -> int:
        ...

    @version.setter
    def version(self, value: int) -> None:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, name: str, value: str) -> None:
        ...

    @overload
    def __init__(self, name: str, value: str, path: str) -> None:
        ...

    @overload
    def __init__(self, name: str, value: str, path: str, domain: str) -> None:
        ...

    def equals(self, comparand: typing.Any) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    def to_string(self) -> str:
        ...


class CookieCollection(System.Object, System.Collections.Generic.ICollection[System.Net.Cookie], System.Collections.Generic.IReadOnlyCollection[System.Net.Cookie], System.Collections.ICollection, typing.Iterable[System.Net.Cookie]):
    """This class has no documentation."""

    @property
    def is_read_only(self) -> bool:
        ...

    @property
    def count(self) -> int:
        ...

    @property
    def is_synchronized(self) -> bool:
        ...

    @property
    def sync_root(self) -> System.Object:
        ...

    @overload
    def __getitem__(self, index: int) -> System.Net.Cookie:
        ...

    @overload
    def __getitem__(self, name: str) -> System.Net.Cookie:
        ...

    def __init__(self) -> None:
        ...

    def __iter__(self) -> typing.Iterator[System.Net.Cookie]:
        ...

    @overload
    def add(self, cookie: System.Net.Cookie) -> None:
        ...

    @overload
    def add(self, cookies: System.Net.CookieCollection) -> None:
        ...

    def clear(self) -> None:
        ...

    def contains(self, cookie: System.Net.Cookie) -> bool:
        ...

    @overload
    def copy_to(self, array: System.Array, index: int) -> None:
        ...

    @overload
    def copy_to(self, array: typing.List[System.Net.Cookie], index: int) -> None:
        ...

    def get_enumerator(self) -> System.Collections.IEnumerator:
        ...

    def remove(self, cookie: System.Net.Cookie) -> bool:
        ...


class CookieContainer(System.Object):
    """This class has no documentation."""

    DEFAULT_COOKIE_LIMIT: int = 300

    DEFAULT_PER_DOMAIN_COOKIE_LIMIT: int = 20

    DEFAULT_COOKIE_LENGTH_LIMIT: int = 4096

    @property
    def capacity(self) -> int:
        ...

    @capacity.setter
    def capacity(self, value: int) -> None:
        ...

    @property
    def count(self) -> int:
        ...

    @property
    def max_cookie_size(self) -> int:
        ...

    @max_cookie_size.setter
    def max_cookie_size(self, value: int) -> None:
        ...

    @property
    def per_domain_capacity(self) -> int:
        ...

    @per_domain_capacity.setter
    def per_domain_capacity(self, value: int) -> None:
        ...

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, capacity: int) -> None:
        ...

    @overload
    def __init__(self, capacity: int, per_domain_capacity: int, max_cookie_size: int) -> None:
        ...

    @overload
    def add(self, cookie: System.Net.Cookie) -> None:
        ...

    @overload
    def add(self, cookies: System.Net.CookieCollection) -> None:
        ...

    @overload
    def add(self, uri: System.Uri, cookie: System.Net.Cookie) -> None:
        ...

    @overload
    def add(self, uri: System.Uri, cookies: System.Net.CookieCollection) -> None:
        ...

    def get_all_cookies(self) -> System.Net.CookieCollection:
        ...

    def get_cookie_header(self, uri: System.Uri) -> str:
        ...

    def get_cookies(self, uri: System.Uri) -> System.Net.CookieCollection:
        ...

    def set_cookies(self, uri: System.Uri, cookie_header: str) -> None:
        ...


class PathList(System.Object):
    """This class has no documentation."""


class HttpVersion(System.Object):
    """This class has no documentation."""

    UNKNOWN: System.Version = ...

    VERSION_10: System.Version = ...

    VERSION_11: System.Version = ...

    VERSION_20: System.Version = ...

    VERSION_30: System.Version = ...


class HttpStatusCode(IntEnum):
    """This class has no documentation."""

    CONTINUE = 100

    SWITCHING_PROTOCOLS = 101

    PROCESSING = 102

    EARLY_HINTS = 103

    OK = 200

    CREATED = 201

    ACCEPTED = 202

    NON_AUTHORITATIVE_INFORMATION = 203

    NO_CONTENT = 204

    RESET_CONTENT = 205

    PARTIAL_CONTENT = 206

    MULTI_STATUS = 207

    ALREADY_REPORTED = 208

    IM_USED = 226

    MULTIPLE_CHOICES = 300

    AMBIGUOUS = 300

    MOVED_PERMANENTLY = 301

    MOVED = 301

    FOUND = 302

    REDIRECT = 302

    SEE_OTHER = 303

    REDIRECT_METHOD = 303

    NOT_MODIFIED = 304

    USE_PROXY = 305

    UNUSED = 306

    TEMPORARY_REDIRECT = 307

    REDIRECT_KEEP_VERB = 307

    PERMANENT_REDIRECT = 308

    BAD_REQUEST = 400

    UNAUTHORIZED = 401

    PAYMENT_REQUIRED = 402

    FORBIDDEN = 403

    NOT_FOUND = 404

    METHOD_NOT_ALLOWED = 405

    NOT_ACCEPTABLE = 406

    PROXY_AUTHENTICATION_REQUIRED = 407

    REQUEST_TIMEOUT = 408

    CONFLICT = 409

    GONE = 410

    LENGTH_REQUIRED = 411

    PRECONDITION_FAILED = 412

    REQUEST_ENTITY_TOO_LARGE = 413

    REQUEST_URI_TOO_LONG = 414

    UNSUPPORTED_MEDIA_TYPE = 415

    REQUESTED_RANGE_NOT_SATISFIABLE = 416

    EXPECTATION_FAILED = 417

    MISDIRECTED_REQUEST = 421

    UNPROCESSABLE_ENTITY = 422

    UNPROCESSABLE_CONTENT = 422

    LOCKED = 423

    FAILED_DEPENDENCY = 424

    UPGRADE_REQUIRED = 426

    PRECONDITION_REQUIRED = 428

    TOO_MANY_REQUESTS = 429

    REQUEST_HEADER_FIELDS_TOO_LARGE = 431

    UNAVAILABLE_FOR_LEGAL_REASONS = 451

    INTERNAL_SERVER_ERROR = 500

    NOT_IMPLEMENTED = 501

    BAD_GATEWAY = 502

    SERVICE_UNAVAILABLE = 503

    GATEWAY_TIMEOUT = 504

    HTTP_VERSION_NOT_SUPPORTED = 505

    VARIANT_ALSO_NEGOTIATES = 506

    INSUFFICIENT_STORAGE = 507

    LOOP_DETECTED = 508

    NOT_EXTENDED = 510

    NETWORK_AUTHENTICATION_REQUIRED = 511


class EndPoint(System.Object, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    def address_family(self) -> System.Net.Sockets.AddressFamily:
        ...

    def create(self, socket_address: System.Net.SocketAddress) -> System.Net.EndPoint:
        ...

    def serialize(self) -> System.Net.SocketAddress:
        ...


class DnsEndPoint(System.Net.EndPoint):
    """This class has no documentation."""

    @property
    def host(self) -> str:
        ...

    @property
    def address_family(self) -> System.Net.Sockets.AddressFamily:
        ...

    @property
    def port(self) -> int:
        ...

    @overload
    def __init__(self, host: str, port: int) -> None:
        ...

    @overload
    def __init__(self, host: str, port: int, address_family: System.Net.Sockets.AddressFamily) -> None:
        ...

    def equals(self, comparand: typing.Any) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    def to_string(self) -> str:
        ...


class CredentialCache(System.Object, System.Net.ICredentials, System.Net.ICredentialsByHost, System.Collections.IEnumerable):
    """This class has no documentation."""

    DEFAULT_CREDENTIALS: System.Net.ICredentials

    DEFAULT_NETWORK_CREDENTIALS: System.Net.NetworkCredential

    def __init__(self) -> None:
        ...

    @overload
    def add(self, uri_prefix: System.Uri, auth_type: str, cred: System.Net.NetworkCredential) -> None:
        ...

    @overload
    def add(self, host: str, port: int, authentication_type: str, credential: System.Net.NetworkCredential) -> None:
        ...

    @overload
    def get_credential(self, uri_prefix: System.Uri, auth_type: str) -> System.Net.NetworkCredential:
        ...

    @overload
    def get_credential(self, host: str, port: int, authentication_type: str) -> System.Net.NetworkCredential:
        ...

    def get_enumerator(self) -> System.Collections.IEnumerator:
        ...

    @overload
    def remove(self, uri_prefix: System.Uri, auth_type: str) -> None:
        ...

    @overload
    def remove(self, host: str, port: int, authentication_type: str) -> None:
        ...


class IPAddress(System.Object, System.ISpanFormattable, System.ISpanParsable[System_Net_IPAddress], System.IUtf8SpanFormattable, System.IUtf8SpanParsable[System_Net_IPAddress]):
    """This class has no documentation."""

    ANY: System.Net.IPAddress = ...

    LOOPBACK: System.Net.IPAddress = ...

    BROADCAST: System.Net.IPAddress = ...

    NONE: System.Net.IPAddress = ...

    I_PV_6_ANY: System.Net.IPAddress = ...

    I_PV_6_LOOPBACK: System.Net.IPAddress = ...

    I_PV_6_NONE: System.Net.IPAddress = ...

    @property
    def address_family(self) -> System.Net.Sockets.AddressFamily:
        ...

    @property
    def scope_id(self) -> int:
        ...

    @scope_id.setter
    def scope_id(self, value: int) -> None:
        ...

    @property
    def is_i_pv_6_multicast(self) -> bool:
        ...

    @property
    def is_i_pv_6_link_local(self) -> bool:
        ...

    @property
    def is_i_pv_6_site_local(self) -> bool:
        ...

    @property
    def is_i_pv_6_teredo(self) -> bool:
        ...

    @property
    def is_i_pv_6_unique_local(self) -> bool:
        ...

    @property
    def is_i_pv_4_mapped_to_i_pv_6(self) -> bool:
        ...

    @property
    def address(self) -> int:
        warnings.warn("IPAddress.Address is address family dependent and has been deprecated. Use IPAddress.Equals to perform comparisons instead.", DeprecationWarning)

    @address.setter
    def address(self, value: int) -> None:
        warnings.warn("IPAddress.Address is address family dependent and has been deprecated. Use IPAddress.Equals to perform comparisons instead.", DeprecationWarning)

    @overload
    def __init__(self, new_address: int) -> None:
        ...

    @overload
    def __init__(self, address: typing.List[int], scopeid: int) -> None:
        ...

    @overload
    def __init__(self, address: System.ReadOnlySpan[int], scopeid: int) -> None:
        ...

    @overload
    def __init__(self, address: typing.List[int]) -> None:
        ...

    @overload
    def __init__(self, address: System.ReadOnlySpan[int]) -> None:
        ...

    def equals(self, comparand: typing.Any) -> bool:
        ...

    def get_address_bytes(self) -> typing.List[int]:
        ...

    def get_hash_code(self) -> int:
        ...

    @staticmethod
    def host_to_network_order(host: int) -> int:
        ...

    @staticmethod
    def is_loopback(address: System.Net.IPAddress) -> bool:
        ...

    @staticmethod
    def is_valid(ip_span: System.ReadOnlySpan[str]) -> bool:
        ...

    @staticmethod
    def is_valid_utf_8(utf_8_text: System.ReadOnlySpan[int]) -> bool:
        ...

    def map_to_i_pv_4(self) -> System.Net.IPAddress:
        ...

    def map_to_i_pv_6(self) -> System.Net.IPAddress:
        ...

    @staticmethod
    def network_to_host_order(network: int) -> int:
        ...

    @staticmethod
    @overload
    def parse(ip_string: str) -> System.Net.IPAddress:
        ...

    @staticmethod
    @overload
    def parse(utf_8_text: System.ReadOnlySpan[int]) -> System.Net.IPAddress:
        ...

    @staticmethod
    @overload
    def parse(ip_span: System.ReadOnlySpan[str]) -> System.Net.IPAddress:
        ...

    def to_string(self) -> str:
        ...

    @overload
    def try_format(self, destination: System.Span[str], chars_written: typing.Optional[int]) -> typing.Tuple[bool, int]:
        ...

    @overload
    def try_format(self, utf_8_destination: System.Span[int], bytes_written: typing.Optional[int]) -> typing.Tuple[bool, int]:
        ...

    @staticmethod
    @overload
    def try_parse(ip_string: str, address: typing.Optional[System.Net.IPAddress]) -> typing.Tuple[bool, System.Net.IPAddress]:
        ...

    @staticmethod
    @overload
    def try_parse(utf_8_text: System.ReadOnlySpan[int], result: typing.Optional[System.Net.IPAddress]) -> typing.Tuple[bool, System.Net.IPAddress]:
        ...

    @staticmethod
    @overload
    def try_parse(ip_span: System.ReadOnlySpan[str], address: typing.Optional[System.Net.IPAddress]) -> typing.Tuple[bool, System.Net.IPAddress]:
        ...

    def try_write_bytes(self, destination: System.Span[int], bytes_written: typing.Optional[int]) -> typing.Tuple[bool, int]:
        ...


class IPNetwork(System.IEquatable[System_Net_IPNetwork], System.ISpanFormattable, System.ISpanParsable[System_Net_IPNetwork], System.IUtf8SpanFormattable, System.IUtf8SpanParsable[System_Net_IPNetwork]):
    """This class has no documentation."""

    @property
    def base_address(self) -> System.Net.IPAddress:
        ...

    @property
    def prefix_length(self) -> int:
        ...

    def __eq__(self, right: System.Net.IPNetwork) -> bool:
        ...

    def __init__(self, base_address: System.Net.IPAddress, prefix_length: int) -> None:
        ...

    def __ne__(self, right: System.Net.IPNetwork) -> bool:
        ...

    def contains(self, address: System.Net.IPAddress) -> bool:
        ...

    @overload
    def equals(self, obj: typing.Any) -> bool:
        ...

    @overload
    def equals(self, other: System.Net.IPNetwork) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    @staticmethod
    @overload
    def parse(s: str) -> System.Net.IPNetwork:
        ...

    @staticmethod
    @overload
    def parse(s: System.ReadOnlySpan[str]) -> System.Net.IPNetwork:
        ...

    @staticmethod
    @overload
    def parse(utf_8_text: System.ReadOnlySpan[int]) -> System.Net.IPNetwork:
        ...

    def to_string(self) -> str:
        ...

    @overload
    def try_format(self, destination: System.Span[str], chars_written: typing.Optional[int]) -> typing.Tuple[bool, int]:
        ...

    @overload
    def try_format(self, utf_8_destination: System.Span[int], bytes_written: typing.Optional[int]) -> typing.Tuple[bool, int]:
        ...

    @staticmethod
    @overload
    def try_parse(s: str, result: typing.Optional[System.Net.IPNetwork]) -> typing.Tuple[bool, System.Net.IPNetwork]:
        ...

    @staticmethod
    @overload
    def try_parse(s: System.ReadOnlySpan[str], result: typing.Optional[System.Net.IPNetwork]) -> typing.Tuple[bool, System.Net.IPNetwork]:
        ...

    @staticmethod
    @overload
    def try_parse(utf_8_text: System.ReadOnlySpan[int], result: typing.Optional[System.Net.IPNetwork]) -> typing.Tuple[bool, System.Net.IPNetwork]:
        ...


class CookieVariant(IntEnum):
    """This class has no documentation."""

    UNKNOWN = 0

    PLAIN = 1

    RFC_2109 = 2

    RFC_2965 = 3

    DEFAULT = ...


class IPEndPoint(System.Net.EndPoint, System.ISpanFormattable, System.ISpanParsable[System_Net_IPEndPoint], System.IUtf8SpanFormattable, System.IUtf8SpanParsable[System_Net_IPEndPoint]):
    """This class has no documentation."""

    MIN_PORT: int = ...

    MAX_PORT: int = ...

    @property
    def address_family(self) -> System.Net.Sockets.AddressFamily:
        ...

    @property
    def address(self) -> System.Net.IPAddress:
        ...

    @address.setter
    def address(self, value: System.Net.IPAddress) -> None:
        ...

    @property
    def port(self) -> int:
        ...

    @port.setter
    def port(self, value: int) -> None:
        ...

    @overload
    def __init__(self, address: int, port: int) -> None:
        ...

    @overload
    def __init__(self, address: System.Net.IPAddress, port: int) -> None:
        ...

    def create(self, socket_address: System.Net.SocketAddress) -> System.Net.EndPoint:
        ...

    def equals(self, comparand: typing.Any) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    @staticmethod
    @overload
    def parse(s: str) -> System.Net.IPEndPoint:
        ...

    @staticmethod
    @overload
    def parse(s: System.ReadOnlySpan[str]) -> System.Net.IPEndPoint:
        ...

    @staticmethod
    @overload
    def parse(utf_8_text: System.ReadOnlySpan[int]) -> System.Net.IPEndPoint:
        ...

    def serialize(self) -> System.Net.SocketAddress:
        ...

    def to_string(self) -> str:
        ...

    @overload
    def try_format(self, destination: System.Span[str], chars_written: typing.Optional[int]) -> typing.Tuple[bool, int]:
        ...

    @overload
    def try_format(self, utf_8_destination: System.Span[int], bytes_written: typing.Optional[int]) -> typing.Tuple[bool, int]:
        ...

    @staticmethod
    @overload
    def try_parse(s: str, result: typing.Optional[System.Net.IPEndPoint]) -> typing.Tuple[bool, System.Net.IPEndPoint]:
        ...

    @staticmethod
    @overload
    def try_parse(s: System.ReadOnlySpan[str], result: typing.Optional[System.Net.IPEndPoint]) -> typing.Tuple[bool, System.Net.IPEndPoint]:
        ...

    @staticmethod
    @overload
    def try_parse(utf_8_text: System.ReadOnlySpan[int], result: typing.Optional[System.Net.IPEndPoint]) -> typing.Tuple[bool, System.Net.IPEndPoint]:
        ...


class CookieException(System.FormatException, System.Runtime.Serialization.ISerializable):
    """This class has no documentation."""

    @overload
    def __init__(self) -> None:
        ...

    @overload
    def __init__(self, message: str) -> None:
        ...

    @overload
    def __init__(self, message: str, inner_exception: System.Exception) -> None:
        ...

    @overload
    def __init__(self, serialization_info: System.Runtime.Serialization.SerializationInfo, streaming_context: System.Runtime.Serialization.StreamingContext) -> None:
        ...

    def get_object_data(self, serialization_info: System.Runtime.Serialization.SerializationInfo, streaming_context: System.Runtime.Serialization.StreamingContext) -> None:
        warnings.warn("Obsoletions.LegacyFormatterImplMessage", DeprecationWarning)


class AuthenticationSchemes(IntEnum):
    """This class has no documentation."""

    NONE = ...

    DIGEST = ...

    NEGOTIATE = ...

    NTLM = ...

    BASIC = ...

    ANONYMOUS = ...

    INTEGRATED_WINDOWS_AUTHENTICATION = ...





from typing import overload
from enum import IntEnum
import typing

import System
import System.Buffers
import System.Text.Unicode


class Utf8(System.Object):
    """This class has no documentation."""

    @staticmethod
    def from_utf_16(source: System.ReadOnlySpan[str], destination: System.Span[int], chars_read: typing.Optional[int], bytes_written: typing.Optional[int], replace_invalid_sequences: bool = True, is_final_block: bool = True) -> typing.Tuple[System.Buffers.OperationStatus, int, int]:
        ...

    @staticmethod
    def is_valid(value: System.ReadOnlySpan[int]) -> bool:
        ...

    @staticmethod
    def to_utf_16(source: System.ReadOnlySpan[int], destination: System.Span[str], bytes_read: typing.Optional[int], chars_written: typing.Optional[int], replace_invalid_sequences: bool = True, is_final_block: bool = True) -> typing.Tuple[System.Buffers.OperationStatus, int, int]:
        ...





from typing import overload
from enum import IntEnum
import abc
import typing

import System
import System.Buffers
import System.Runtime.InteropServices

System_Buffers_StandardFormat = typing.Any

System_Buffers_ArrayPool_T = typing.TypeVar("System_Buffers_ArrayPool_T")
System_Buffers_MemoryManager_T = typing.TypeVar("System_Buffers_MemoryManager_T")
System_Buffers_IMemoryOwner_T = typing.TypeVar("System_Buffers_IMemoryOwner_T")
System_Buffers_SearchValues_T = typing.TypeVar("System_Buffers_SearchValues_T")


class StandardFormat(System.IEquatable[System_Buffers_StandardFormat]):
    """This class has no documentation."""

    NO_PRECISION: int = ...

    MAX_PRECISION: int = 99

    @property
    def symbol(self) -> str:
        ...

    @property
    def precision(self) -> int:
        ...

    @property
    def has_precision(self) -> bool:
        ...

    @property
    def is_default(self) -> bool:
        ...

    def __eq__(self, right: System.Buffers.StandardFormat) -> bool:
        ...

    def __init__(self, symbol: str, precision: int = ...) -> None:
        ...

    def __ne__(self, right: System.Buffers.StandardFormat) -> bool:
        ...

    @overload
    def equals(self, obj: typing.Any) -> bool:
        ...

    @overload
    def equals(self, other: System.Buffers.StandardFormat) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    @staticmethod
    @overload
    def parse(format: System.ReadOnlySpan[str]) -> System.Buffers.StandardFormat:
        ...

    @staticmethod
    @overload
    def parse(format: str) -> System.Buffers.StandardFormat:
        ...

    def to_string(self) -> str:
        ...

    @staticmethod
    def try_parse(format: System.ReadOnlySpan[str], result: typing.Optional[System.Buffers.StandardFormat]) -> typing.Tuple[bool, System.Buffers.StandardFormat]:
        ...


class OperationStatus(IntEnum):
    """This class has no documentation."""

    DONE = 0

    DESTINATION_TOO_SMALL = 1

    NEED_MORE_DATA = 2

    INVALID_DATA = 3


class ArrayPool(typing.Generic[System_Buffers_ArrayPool_T], System.Object, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    SHARED: System.Buffers.ArrayPool[System_Buffers_ArrayPool_T]

    @staticmethod
    @overload
    def create() -> System.Buffers.ArrayPool[System_Buffers_ArrayPool_T]:
        ...

    @staticmethod
    @overload
    def create(max_array_length: int, max_arrays_per_bucket: int) -> System.Buffers.ArrayPool[System_Buffers_ArrayPool_T]:
        ...

    def rent(self, minimum_length: int) -> typing.List[System_Buffers_ArrayPool_T]:
        ...

    def Return(self, array: typing.List[System_Buffers_ArrayPool_T], clearArray: bool = False) -> None:
        ...


class MemoryHandle(System.IDisposable):
    """This class has no documentation."""

    @property
    def pointer(self) -> typing.Any:
        ...

    def __init__(self, pointer: typing.Any, handle: System.Runtime.InteropServices.GCHandle = ..., pinnable: System.Buffers.IPinnable = ...) -> None:
        ...

    def dispose(self) -> None:
        ...


class IPinnable(metaclass=abc.ABCMeta):
    """This class has no documentation."""

    def pin(self, element_index: int) -> System.Buffers.MemoryHandle:
        ...

    def unpin(self) -> None:
        ...


class MemoryManager(typing.Generic[System_Buffers_MemoryManager_T], System.Object, System.Buffers.IMemoryOwner[System_Buffers_MemoryManager_T], System.Buffers.IPinnable, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    def memory(self) -> System.Memory[System_Buffers_MemoryManager_T]:
        ...

    @overload
    def create_memory(self, length: int) -> System.Memory[System_Buffers_MemoryManager_T]:
        ...

    @overload
    def create_memory(self, start: int, length: int) -> System.Memory[System_Buffers_MemoryManager_T]:
        ...

    def dispose(self, disposing: bool) -> None:
        ...

    def get_span(self) -> System.Span[System_Buffers_MemoryManager_T]:
        ...

    def pin(self, element_index: int = 0) -> System.Buffers.MemoryHandle:
        ...

    def unpin(self) -> None:
        ...


class IMemoryOwner(typing.Generic[System_Buffers_IMemoryOwner_T], System.IDisposable, metaclass=abc.ABCMeta):
    """This class has no documentation."""

    @property
    @abc.abstractmethod
    def memory(self) -> System.Memory[System_Buffers_IMemoryOwner_T]:
        ...


class SearchValues(typing.Generic[System_Buffers_SearchValues_T], System.Object):
    """This class has no documentation."""

    def contains(self, value: System_Buffers_SearchValues_T) -> bool:
        ...

    @staticmethod
    @overload
    def create(*values: typing.Union[int, typing.Iterable[int]]) -> System.Buffers.SearchValues[int]:
        ...

    @staticmethod
    @overload
    def create(*values: typing.Union[str, typing.Iterable[str]]) -> System.Buffers.SearchValues[str]:
        ...

    @staticmethod
    @overload
    def create(values: System.ReadOnlySpan[str], comparison_type: System.StringComparison) -> System.Buffers.SearchValues[str]:
        ...





from typing import overload
from enum import IntEnum
import abc
import typing

import QuantConnect.Exceptions
import System
import System.Reflection


class IExceptionInterpreter(metaclass=abc.ABCMeta):
    """Defines an exception interpreter. Interpretations are invoked on IAlgorithm.RunTimeError"""

    @property
    @abc.abstractmethod
    def order(self) -> int:
        """Determines the order that a class that implements this interface should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception.
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        This provides a link back allowing the inner exceptions to be interpreted using the interpreters
        configured in the IExceptionInterpreter. Individual implementations *may* ignore
        this value if required.
        :returns: The interpreted exception.
        """
        ...


class SystemExceptionInterpreter(System.Object, QuantConnect.Exceptions.IExceptionInterpreter):
    """Base handler that will try get an exception file and line"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception. f
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        :returns: The interpreted exception.
        """
        ...

    @staticmethod
    def try_get_line_and_file(stack_trace: str, file_and_line: typing.Optional[str]) -> typing.Tuple[bool, str]:
        """Helper method to get the file and line from a C# stacktrace"""
        ...


class ClrBubbledExceptionInterpreter(QuantConnect.Exceptions.SystemExceptionInterpreter):
    """Interprets ClrBubbledException instances"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception. f
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        :returns: The interpreted exception.
        """
        ...


class ScheduledEventExceptionInterpreter(System.Object, QuantConnect.Exceptions.IExceptionInterpreter):
    """Interprets ScheduledEventException instances"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception.
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        This provides a link back allowing the inner exceptions to be interpreted using the interpreters
        configured in the IExceptionInterpreter. Individual implementations *may* ignore
        this value if required.
        :returns: The interpreted exception.
        """
        ...


class PythonExceptionInterpreter(System.Object, QuantConnect.Exceptions.IExceptionInterpreter):
    """Interprets PythonExceptionInterpreter instances"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception. f
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        :returns: The interpreted exception.
        """
        ...


class InvalidTokenPythonExceptionInterpreter(QuantConnect.Exceptions.PythonExceptionInterpreter):
    """Interprets InvalidTokenPythonExceptionInterpreter instances"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception.
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        :returns: The interpreted exception.
        """
        ...


class NoMethodMatchPythonExceptionInterpreter(QuantConnect.Exceptions.PythonExceptionInterpreter):
    """Interprets NoMethodMatchPythonExceptionInterpreter instances"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception.
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        :returns: The interpreted exception.
        """
        ...


class DllNotFoundPythonExceptionInterpreter(System.Object, QuantConnect.Exceptions.IExceptionInterpreter):
    """Interprets DllNotFoundPythonExceptionInterpreter instances"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception.
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        :returns: The interpreted exception.
        """
        ...


class StackExceptionInterpreter(System.Object, QuantConnect.Exceptions.IExceptionInterpreter):
    """Interprets exceptions using the configured interpretations"""

    INSTANCE: System.Lazy[QuantConnect.Exceptions.StackExceptionInterpreter] = ...
    """Stack interpreter instance"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    @property
    def interpreters(self) -> typing.Iterable[QuantConnect.Exceptions.IExceptionInterpreter]:
        """Gets the interpreters loaded into this instance"""
        ...

    def __init__(self, interpreters: typing.List[QuantConnect.Exceptions.IExceptionInterpreter]) -> None:
        """
        Initializes a new instance of the StackExceptionInterpreter class
        
        :param interpreters: The interpreters to use
        """
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception.
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    @staticmethod
    def create_from_assemblies(assemblies: typing.List[System.Reflection.Assembly]) -> QuantConnect.Exceptions.StackExceptionInterpreter:
        """
        Creates a new StackExceptionInterpreter by loading implementations with default constructors from the specified assemblies
        
        :param assemblies: The assemblies to scan
        :returns: A new StackExceptionInterpreter containing interpreters from the specified assemblies.
        """
        ...

    def get_exception_message_header(self, exception: System.Exception) -> str:
        """
        Combines the exception messages from this exception and all inner exceptions.
        
        :param exception: The exception to create a collated message from
        :returns: The collate exception message.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter = None) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        This provides a link back allowing the inner exceptions to be interpreted using the intepretators
        configured in the StackExceptionInterpreter. Individual implementations *may* ignore
        this value if required.
        :returns: The interpreted exception.
        """
        ...


class UnsupportedOperandPythonExceptionInterpreter(QuantConnect.Exceptions.PythonExceptionInterpreter):
    """Interprets UnsupportedOperandPythonExceptionInterpreter instances"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception.
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        :returns: The interpreted exception.
        """
        ...


class KeyErrorPythonExceptionInterpreter(QuantConnect.Exceptions.PythonExceptionInterpreter):
    """Interprets KeyErrorPythonExceptionInterpreter instances"""

    @property
    def order(self) -> int:
        """Determines the order that an instance of this class should be called"""
        ...

    def can_interpret(self, exception: System.Exception) -> bool:
        """
        Determines if this interpreter should be applied to the specified exception.
        
        :param exception: The exception to check
        :returns: True if the exception can be interpreted, false otherwise.
        """
        ...

    def interpret(self, exception: System.Exception, inner_interpreter: QuantConnect.Exceptions.IExceptionInterpreter) -> System.Exception:
        """
        Interprets the specified exception into a new exception
        
        :param exception: The exception to be interpreted
        :param inner_interpreter: An interpreter that should be applied to the inner exception.
        :returns: The interpreted exception.
        """
        ...





from typing import overload
from enum import IntEnum
import typing

import System
import System.Runtime.Versioning

System_Runtime_Versioning_FrameworkName = typing.Any


class ComponentGuaranteesOptions(IntEnum):
    """This class has no documentation."""

    NONE = 0

    EXCHANGE = ...

    STABLE = ...

    SIDE_BY_SIDE = ...


class ComponentGuaranteesAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def guarantees(self) -> System.Runtime.Versioning.ComponentGuaranteesOptions:
        ...

    def __init__(self, guarantees: System.Runtime.Versioning.ComponentGuaranteesOptions) -> None:
        ...


class TargetFrameworkAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def framework_name(self) -> str:
        ...

    @property
    def framework_display_name(self) -> str:
        ...

    @framework_display_name.setter
    def framework_display_name(self, value: str) -> None:
        ...

    def __init__(self, framework_name: str) -> None:
        ...


class ResourceScope(IntEnum):
    """This class has no documentation."""

    NONE = 0

    MACHINE = ...

    PROCESS = ...

    APP_DOMAIN = ...

    LIBRARY = ...

    PRIVATE = ...

    ASSEMBLY = ...


class ResourceExposureAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def resource_exposure_level(self) -> System.Runtime.Versioning.ResourceScope:
        ...

    def __init__(self, exposure_level: System.Runtime.Versioning.ResourceScope) -> None:
        ...


class ResourceConsumptionAttribute(System.Attribute):
    """This class has no documentation."""

    @property
    def resource_scope(self) -> System.Runtime.Versioning.ResourceScope:
        ...

    @property
    def consumption_scope(self) -> System.Runtime.Versioning.ResourceScope:
        ...

    @overload
    def __init__(self, resource_scope: System.Runtime.Versioning.ResourceScope) -> None:
        ...

    @overload
    def __init__(self, resource_scope: System.Runtime.Versioning.ResourceScope, consumption_scope: System.Runtime.Versioning.ResourceScope) -> None:
        ...


class FrameworkName(System.Object, System.IEquatable[System_Runtime_Versioning_FrameworkName]):
    """This class has no documentation."""

    @property
    def identifier(self) -> str:
        ...

    @property
    def version(self) -> System.Version:
        ...

    @property
    def profile(self) -> str:
        ...

    @property
    def full_name(self) -> str:
        ...

    def __eq__(self, right: System.Runtime.Versioning.FrameworkName) -> bool:
        ...

    @overload
    def __init__(self, identifier: str, version: System.Version) -> None:
        ...

    @overload
    def __init__(self, identifier: str, version: System.Version, profile: str) -> None:
        ...

    @overload
    def __init__(self, framework_name: str) -> None:
        ...

    def __ne__(self, right: System.Runtime.Versioning.FrameworkName) -> bool:
        ...

    @overload
    def equals(self, obj: typing.Any) -> bool:
        ...

    @overload
    def equals(self, other: System.Runtime.Versioning.FrameworkName) -> bool:
        ...

    def get_hash_code(self) -> int:
        ...

    def to_string(self) -> str:
        ...


class VersioningHelper(System.Object):
    """This class has no documentation."""

    @staticmethod
    @overload
    def make_version_safe_name(name: str, _from: System.Runtime.Versioning.ResourceScope, to: System.Runtime.Versioning.ResourceScope) -> str:
        ...

    @staticmethod
    @overload
    def make_version_safe_name(name: str, _from: System.Runtime.Versioning.ResourceScope, to: System.Runtime.Versioning.ResourceScope, type: typing.Type) -> str:
        ...


